@@ -5,39 +5,21 @@ from collections import defaultdict
 
 from grapheekdb.lib.undef import UNDEFINED
 from grapheekdb.lib.readwrite import GraphReadWrite
+from grapheekdb.lib.validations import check_valid_data
+from grapheekdb.lib.exceptions import GrapheekDataException, GrapheekDataPreparationFailedException, GrapheekIndexAlreadyExistsException
+from grapheekdb.lib.exceptions import GrapheekIndexCreationFailedException, GrapheekIndexRemovalFailedException, GrapheekNoSuchTraversalException
+from grapheekdb.lib.exceptions import GrapheekMissingKeyException, GrapheekInvalidDataTypeException
+from grapheekdb.lib.exceptions import GrapheekUnknownScriptException, GrapheekUnknownAlias
+from grapheekdb.lib.expressions import eval_expr
 
-from grapheekdb.backends.data.keys import PREPARED
+from grapheekdb.backends.data.keys import build_key, PREPARED
 from grapheekdb.backends.data.keys import METADATA_EDGE_COUNTER, METADATA_EDGE_INDEX_COUNTER, METADATA_EDGE_INDEX_LIST, METADATA_EDGE_INDEX_FIELDS_PREFIX, METADATA_EDGE_INDEX_PREFIX
 from grapheekdb.backends.data.keys import METADATA_VERTEX_COUNTER, METADATA_VERTEX_INDEX_COUNTER, METADATA_VERTEX_INDEX_LIST, METADATA_VERTEX_INDEX_FIELDS_PREFIX, METADATA_VERTEX_INDEX_PREFIX
 from grapheekdb.backends.data.keys import METADATA_VERTEX_REMOVED_COUNTER, METADATA_EDGE_REMOVED_COUNTER
-from grapheekdb.backends.data.keys import build_key
-from grapheekdb.backends.data.keys import DATA_SUFFIX
-from grapheekdb.backends.data.keys import IN_EDGES_SUFFIX, IN_VERTICES_SUFFIX
-from grapheekdb.backends.data.keys import OUT_EDGES_SUFFIX, OUT_VERTICES_SUFFIX
-from grapheekdb.backends.data.keys import BOTH_EDGES_SUFFIX, BOTH_VERTICES_SUFFIX
-from grapheekdb.backends.data.keys import COUNT_SUFFIX
-from grapheekdb.backends.data.keys import KIND_VERTEX, KIND_EDGE, KIND_INDEX
-#from grapheekdb.backends.data.keys import VERTEX_DATA_REGEXP, EDGE_DATA_REGEXP
-#from grapheekdb.backends.data.keys import VERTEX_DATA_PATTERN, EDGE_DATA_PATTERN
-from grapheekdb.backends.data.keys import METADATA_EDGE_ID_LIST_PREFIX, METADATA_VERTEX_ID_LIST_PREFIX
-from grapheekdb.backends.data.keys import CHUNK_SIZE
-
-from grapheekdb.lib.validations import check_valid_data
-
-from grapheekdb.lib.exceptions import GrapheekDataException
-from grapheekdb.lib.exceptions import GrapheekDataPreparationFailedException
-from grapheekdb.lib.exceptions import GrapheekIndexAlreadyExistsException
-from grapheekdb.lib.exceptions import GrapheekIndexCreationFailedException
-from grapheekdb.lib.exceptions import GrapheekIndexRemovalFailedException
-from grapheekdb.lib.exceptions import GrapheekNoSuchTraversalException
-from grapheekdb.lib.exceptions import GrapheekMissingKeyException
-from grapheekdb.lib.exceptions import GrapheekInvalidDataTypeException
-from grapheekdb.lib.exceptions import GrapheekUnknownScriptException
-from grapheekdb.lib.exceptions import GrapheekUnknownAlias
-
-from grapheekdb.backends.data.filtertools import build_filter_funcs
-from grapheekdb.backends.data.filtertools import filter_entities
-
+from grapheekdb.backends.data.keys import DATA_SUFFIX, IN_EDGES_SUFFIX, IN_VERTICES_SUFFIX, OUT_EDGES_SUFFIX, OUT_VERTICES_SUFFIX, BOTH_EDGES_SUFFIX, BOTH_VERTICES_SUFFIX
+from grapheekdb.backends.data.keys import COUNT_SUFFIX, KIND_VERTEX, KIND_EDGE, KIND_INDEX
+from grapheekdb.backends.data.keys import METADATA_EDGE_ID_LIST_PREFIX, METADATA_VERTEX_ID_LIST_PREFIX, CHUNK_SIZE
+from grapheekdb.backends.data.filtertools import build_filter_funcs, filter_entities
 from grapheekdb.backends.data.indexes import ExactIndex, normalize_value
 from grapheekdb.backends.data.operations import Addition, Removal
 from grapheekdb.backends.data.optimizer import Optimizer
@@ -163,13 +145,15 @@ class DotExport(object):
 
 class EntityAggregate(object):
 
-    def __init__(self, graph, entity_kind, entity_iterator, pair_iterator=None, agg='+', asc=False):
+    def __init__(self, graph, entity_kind, entity_iterator, pair_iterator=None, agg='+', expr=None, asc=False):
         self._graph = graph
         self._entity_kind = entity_kind
         self._entity_iterator = entity_iterator
         self._pair_iterator = pair_iterator
+        assert(agg in (None, '+', '%'))
         self._agg = agg
-        self._asc = asc
+        self._expr = expr
+        self._mult = 1 if asc else -1
         # -------------------------------
         self._current_iteration = None
 
@@ -179,43 +163,36 @@ class EntityAggregate(object):
         # so we'll just iterate on _pair_iterator
         if self._agg is None:
             return list(self._pair_iterator)
-        assert(len(self._agg) > 0)
-        agg_type = self._agg[0]
-        assert(agg_type in ['+', '%'])
-        field_path = self._agg[1:]
-        try:
-            alias, field_name = field_path.split('.')
-        except:
-            alias = None
-            field_name = field_path
-        mult = 1 if self._asc else -1
         if self._agg == '+':
             counters = defaultdict(int)
         else:
             counters = defaultdict(float)
         total = 0
-        entity_cache = {}
+        entity_data_cache = {}
         for entity_id in self._entity_iterator._iterate(on_item=False):
-            if field_name == '':
+            if self._expr is None:
                 value = 1
             else:
-                if alias is None:
-                    obj = entity = entity_cache.get(entity_id, self._graph._item_from_id(self._entity_kind, entity_id))
-                    entity_cache[entity_id] = entity
-                else:
-                    obj = self._entity_iterator._context[alias]
-                value = getattr(obj, field_name, 0)
+                # Safe eval of expression :
+                entity_data = entity_data_cache.get(entity_id, None)
+                if entity_data is None:
+                    entity_data = self._graph._item_from_id(self._entity_kind, entity_id).data()
+                    entity_data_cache[entity_id] = entity_data
+                context = entity_data
+                context['_'] = self._entity_iterator._context
+                try:
+                    value = eval_expr(context, self._expr)
+                finally:
+                    del context['_']
             counters[entity_id] += value
             total += value
-        if total and (agg_type == '%'):
+        if total and (self._agg == '%'):
             for entity_id, value in counters.items():
                 counters[entity_id] = float(counters[entity_id]) / float(total)  # This may raise an exception, it's up to the user to handle it...
-        ordered_counters = sorted(counters.items(), key=lambda t: t[1] * mult)
+        ordered_counters = sorted(counters.items(), key=lambda t: t[1] * self._mult)
         results = []
         for entity_id, count in ordered_counters:
-            entity = entity_cache.get(entity_id, None)
-            if entity is None:
-                entity = self._graph._item_from_id(self._entity_kind, entity_id)
+            entity = self._graph._item_from_id(self._entity_kind, entity_id)
             results.append((entity, count))
         return results
 
@@ -488,8 +465,11 @@ class EntityIterator(object):
             result.append([self._context.get(alias, None) for alias in aliases])
         return result
 
-    def agg(self, agg='+', asc=False):
-        return EntityAggregate(self._graph, self._src_kind, self, pair_iterator=None, agg=agg, asc=asc)
+    def sum(self, expr=None, asc=False):
+        return EntityAggregate(self._graph, self._src_kind, self, pair_iterator=None, agg='+', expr=expr, asc=asc)
+
+    def percent(self, expr=None, asc=False):
+        return EntityAggregate(self._graph, self._src_kind, self, pair_iterator=None, agg='%', expr=expr, asc=asc)
 
     def call(self, _fname, *aliases, **params):
         contexts = []
@@ -573,6 +553,12 @@ class Entity(object):
             current_parent = current_iterator
         return current_iterator
 
+    def get(self, field, default=None):
+        try:
+            return self.__getattr__(field)
+        except AttributeError:
+            return default
+
     def inV(self, *args, **filters):
         return self._jump(KIND_VERTEX, IN_VERTICES_SUFFIX, *args, **filters)
 
