@@ -6,7 +6,6 @@ import lookups
 
 from grapheekdb.lib.undef import UNDEFINED
 
-from grapheekdb.backends.data.keys import build_key
 from grapheekdb.backends.data.keys import DATA_SUFFIX
 
 from grapheekdb.lib.exceptions import GrapheekSubLookupNotImplementedException
@@ -25,7 +24,7 @@ def get_exact_filters(**filters):
             exact_filters[field] = value
         elif len_lst == 2:
             field, clause = lst
-            if clause == 'exact':
+            if clause in ['exact', 'in']:
                 exact_filters[field] = value
         else:
             raise GrapheekSubLookupNotImplementedException
@@ -60,40 +59,44 @@ def entity_match(filter_funcs, data):
         # This won't normally occur when load testing with greenlets -> thus no cover
         # (or worst : direct access to backend by another "process")
         return False
-    match = True
     for field, clause, filter_func in filter_funcs:
-        field_value = data.get(field, UNDEFINED)
-        if field_value == UNDEFINED:
-            if clause == 'isnull':
-                field_value = None
-            else:
-                match = False
-                break
+        try:
+            field_value = data[field]
+        except KeyError:
+            field_value = None
+        if field_value is None and clause != 'isnull':
+            return False
         try:
             match = filter_func(field_value)
         except:
             # Whatever happens, it shouldn't
             # So, let's consider that data wasn't ok for the filter
-            match = False
+            return False
         if not(match):
-            break
-    return match
+            return False
+    return True
 
 
 def filter_entities(graph, kind, iterator, filter_funcs, on_item=False):
-    for entity_id in iterator:
-        if filter_funcs:
-            data = graph._get(None, build_key(kind, entity_id, DATA_SUFFIX))
-            match = entity_match(filter_funcs, data)
-            if match:
-                if on_item:
+    if filter_funcs:
+        if on_item:
+            for entity_id in iterator:
+                data = graph._get(None, kind + '/' + str(entity_id) + '/' + DATA_SUFFIX)
+                match = entity_match(filter_funcs, data)
+                if match:
                     yield graph._item_from_id(kind, entity_id)
-                else:
-                    yield entity_id
         else:
-            # I don't do : yield item if on_item else entity_id
-            # Because item is not computed (when on_item is False, this is not useful to instantiate an Entity)
-            if on_item:
+            for entity_id in iterator:
+                data = graph._get(None, kind + '/' + str(entity_id) + '/' + DATA_SUFFIX)
+                match = entity_match(filter_funcs, data)
+                if match:
+                    yield entity_id
+    else:
+        # I don't do : yield item if on_item else entity_id
+        # Because item is not computed (when on_item is False, this is not useful to instantiate an Entity)
+        if on_item:
+            for entity_id in iterator:
                 yield graph._item_from_id(kind, entity_id)
-            else:
+        else:
+            for entity_id in iterator:
                 yield entity_id
