@@ -185,8 +185,12 @@ class ProxyEntityIterator(ProxyIteratorCommon):
     def collect(self, *aliases):
         return self._raw('collect', *aliases)
 
-    def agg(self, agg='+', asc=False):
-        command = ('agg', [], dict(agg=agg, asc=asc))
+    def sum(self, expr=None, asc=False):
+        command = ('sum', [expr], dict(asc=asc))
+        return ProxyEntityAggregate(self._graph, self._commands + [command])
+
+    def percent(self, expr=None, asc=False):
+        command = ('percent', [expr], dict(asc=asc))
         return ProxyEntityAggregate(self._graph, self._commands + [command])
 
     def call(self, _fname, *aliases, **params):
@@ -248,6 +252,12 @@ class ProxyEntity(object):
         response = self._graph._request(self._commands + [command])
         return unmarshall(self._graph, response)
 
+    def get(self, field, default=None):
+        try:
+            return self.__getattr__(field)
+        except AttributeError:
+            return default
+
     def inV(self, *args, **kwargs):
         return self._do('inV', *args, **kwargs)
 
