@@ -83,3 +83,9 @@ class GrapheekUnknownScriptException(GrapheekDataException):
 
 class GrapheekUnknownAlias(GrapheekDataException):
     pass
+
+
+# Expr exceptions :
+
+class GrapheekInvalidExpression(GrapheekDataException):
+    pass
