@@ -52,9 +52,7 @@ def marshall(item):
 
 
 def runserver(_address, _backend, _scripts, _context, **params):
-    backend_parts = _backend.split('.')
-    class_name = backend_parts[-1]
-    class_module_name = '.'.join(backend_parts[:-1])
+    class_module_name, class_name = _backend.rsplit('.', 1)
     # START -- Following lines may raise an exception if data module cannot be imported
     # I'm letting exception propagate so that the user can fix path (or other potential errors)
     __import__(class_module_name)
